@@ -106,9 +106,9 @@ class Project(object):
     def create_storages(self):
         source = self.config['source']
         target = self.config['target']
-        self.source_storage = create_storage(source['type'], source['path'], self.path, self,
+        self.source_storage = create_storage(source['type'], 'source', source['path'], self.path, self,
                                              **source.get('params', {}))
-        self.target_storage = create_storage(target['type'], target['path'], self.path, self,
+        self.target_storage = create_storage(target['type'], 'target', target['path'], self.path, self,
                                              **target.get('params', {}))
 
     def update_storage(self, storage_for, storage_kwargs):
@@ -116,9 +116,9 @@ class Project(object):
         def _update_storage(storage_for, storage_kwargs):
             storage_name = storage_kwargs.pop('name', storage_for)
             storage_type = storage_kwargs.pop('type')
-            # storage_path = storage_kwargs.pop('path', None)
-            storage_path = self.config[storage_for]['path']
-            storage = create_storage(storage_type, storage_path, self.path, self, **storage_kwargs)
+            storage_path = storage_kwargs.pop('path', None)
+            # storage_path = self.config[storage_for]['path']
+            storage = create_storage(storage_type, storage_name, storage_path, self.path, self, **storage_kwargs)
             self.config[storage_for] = {
                 'name': storage_name,
                 'type': storage_type,
@@ -444,6 +444,9 @@ class Project(object):
         self.source_storage.remove(task_id)
         self.delete_completion(task_id)
 
+        self.update_derived_input_schema()
+        self.update_derived_output_schema()
+
     def get_completions_ids(self):
         """ List completion ids from output_dir directory
 
