@@ -54,7 +54,7 @@ def tasks_from_file(filename, file, project):
             path = os.path.join(upload_dir, filename)
             open(path, 'wb').write(data)
             # prepare task
-            tasks = [{'data': {settings.UPLOAD_DATA_UNDEFINED_NAME: HOSTNAME + '/upload/' + filename}}]
+            tasks = [{'data': {settings.UPLOAD_DATA_UNDEFINED_NAME: HOSTNAME + '/data/upload/' + filename}}]
 
     except Exception as exc:
         raise ValidationError('Failed to parse input file ' + filename + ': ' + str(exc))
@@ -160,7 +160,7 @@ def aggregate_tasks(files, project):
     # scan all files
     for filename, file in files.items():
         # extracted file from archive
-        if file is 'archive':
+        if file == 'archive':
             with open(filename) as f:
                 tasks += tasks_from_file(filename, f, project)
         # file from request
